"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Webhook = void 0;
const builders_1 = require("@discordjs/builders");
const axios_1 = __importDefault(require("axios"));
const crypto_1 = require("crypto");
const Twitter_enum_1 = require("../enums/Twitter.enum");
const Limiter_1 = require("../Limiter");
const logger_1 = require("../logger");
const SpaceUtil_1 = require("../utils/SpaceUtil");
const TwitterUtil_1 = require("../utils/TwitterUtil");
const ConfigManager_1 = require("./ConfigManager");
class Webhook {
    constructor(audioSpace, masterUrl) {
        this.audioSpace = audioSpace;
        this.masterUrl = masterUrl;
        const username = SpaceUtil_1.SpaceUtil.getHostUsername(audioSpace);
        const spaceId = SpaceUtil_1.SpaceUtil.getId(audioSpace);
        this.logger = logger_1.logger.child({ label: `[Webhook] [${username}] [${spaceId}]` });
    }
    // eslint-disable-next-line class-methods-use-this
    get config() {
        return ConfigManager_1.configManager.config?.webhooks;
    }
    send() {
        this.sendDiscord();
    }
    async post(url, body) {
        const requestId = (0, crypto_1.randomUUID)();
        try {
            this.logger.debug('--> post', {
                requestId,
                url: url.replace(/.{60}$/, '****'),
                body,
            });
            const { data } = await axios_1.default.post(url, body);
            this.logger.debug('<-- post', { requestId });
            return data;
        }
        catch (error) {
            this.logger.error(`post: ${error.message}`, { requestId });
        }
        return null;
    }
    sendDiscord() {
        this.logger.debug('sendDiscord');
        const configs = Array.from(this.config?.discord || []);
        configs.forEach((config) => {
            if (!config.active) {
                return;
            }
            const urls = Array.from(config.urls || [])
                .filter((v) => v);
            const usernames = Array.from(config.usernames || [])
                .filter((v) => v)
                .map((v) => v.toLowerCase());
            if (!urls.length || !usernames.length) {
                return;
            }
            if (!usernames.find((v) => v === '<all>') && usernames.every((v) => !SpaceUtil_1.SpaceUtil.isParticipant(this.audioSpace, v))) {
                return;
            }
            try {
                // Build content with mentions
                let content = '';
                if (this.audioSpace.metadata.state === Twitter_enum_1.AudioSpaceMetadataState.RUNNING) {
                    Array.from(config.mentions?.roleIds || []).map((v) => v).forEach((roleId) => {
                        content += `<@&${roleId}> `;
                    });
                    Array.from(config.mentions?.userIds || []).map((v) => v).forEach((userId) => {
                        content += `<@${userId}> `;
                    });
                    content = [content, config.startMessage].filter((v) => v).map((v) => v.trim()).join(' ');
                }
                if (this.audioSpace.metadata.state === Twitter_enum_1.AudioSpaceMetadataState.ENDED) {
                    content = [content, config.endMessage].filter((v) => v).map((v) => v.trim()).join(' ');
                }
                content = content.trim();
                // Build request payload
                const payload = {
                    content,
                    embeds: [this.getEmbed(usernames)],
                };
                // Send
                urls.forEach((url) => Limiter_1.discordWebhookLimiter.schedule(() => this.post(url, payload)));
            }
            catch (error) {
                this.logger.error(`sendDiscord: ${error.message}`);
            }
        });
    }
    getEmbedTitle(usernames) {
        const hostUsername = SpaceUtil_1.SpaceUtil.getHostUsername(this.audioSpace);
        const host = (0, builders_1.inlineCode)(hostUsername);
        if (this.audioSpace.metadata.state === Twitter_enum_1.AudioSpaceMetadataState.ENDED) {
            return `${host} Space ended`;
        }
        if (!usernames.some((v) => v.toLowerCase() === hostUsername.toLowerCase())
            && usernames.some((v) => SpaceUtil_1.SpaceUtil.isAdmin(this.audioSpace, v))) {
            const participants = usernames
                .map((v) => SpaceUtil_1.SpaceUtil.getParticipant(this.audioSpace.participants.admins, v))
                .filter((v) => v);
            if (participants.length) {
                const guests = participants
                    .map((v) => (0, builders_1.inlineCode)(v.twitter_screen_name))
                    .join(', ');
                return `${guests} is co-hosting ${host}'s Space`;
            }
        }
        if (usernames.some((v) => SpaceUtil_1.SpaceUtil.isSpeaker(this.audioSpace, v))) {
            const participants = usernames
                .map((v) => SpaceUtil_1.SpaceUtil.getParticipant(this.audioSpace.participants.speakers, v))
                .filter((v) => v);
            if (participants.length) {
                const guests = participants
                    .map((v) => (0, builders_1.inlineCode)(v.twitter_screen_name))
                    .join(', ');
                return `${guests} is speaking in ${host}'s Space`;
            }
        }
        if (usernames.some((v) => SpaceUtil_1.SpaceUtil.isListener(this.audioSpace, v))) {
            const participants = usernames
                .map((v) => SpaceUtil_1.SpaceUtil.getParticipant(this.audioSpace.participants.listeners, v))
                .filter((v) => v);
            if (participants.length) {
                const guests = participants
                    .map((v) => (0, builders_1.inlineCode)(v.twitter_screen_name))
                    .join(', ');
                return `${guests} is listening in ${host}'s Space`;
            }
        }
        return `${host} is hosting a Space`;
    }
    getEmbed(usernames) {
        const username = SpaceUtil_1.SpaceUtil.getHostUsername(this.audioSpace);
        const name = SpaceUtil_1.SpaceUtil.getHostName(this.audioSpace);
        const fields = [
            {
                name: '📄 Title',
                value: (0, builders_1.codeBlock)(SpaceUtil_1.SpaceUtil.getTitle(this.audioSpace)),
            },
        ];
        if ([Twitter_enum_1.AudioSpaceMetadataState.RUNNING, Twitter_enum_1.AudioSpaceMetadataState.ENDED].includes(this.audioSpace.metadata.state)) {
            if (this.audioSpace.metadata.started_at) {
                fields.push({
                    name: '▶️ Started at',
                    value: Webhook.getEmbedLocalTime(this.audioSpace.metadata.started_at),
                    inline: true,
                });
            }
        }
        if ([Twitter_enum_1.AudioSpaceMetadataState.ENDED].includes(this.audioSpace.metadata.state)) {
            if (this.audioSpace.metadata.ended_at) {
                fields.push({
                    name: '⏹️ Ended at',
                    value: Webhook.getEmbedLocalTime(Number(this.audioSpace.metadata.ended_at)),
                    inline: true,
                });
            }
        }
        if ([Twitter_enum_1.AudioSpaceMetadataState.RUNNING, Twitter_enum_1.AudioSpaceMetadataState.ENDED].includes(this.audioSpace.metadata.state)) {
            fields.push({
                name: '🔗 Playlist url',
                value: (0, builders_1.codeBlock)(this.masterUrl),
            });
        }
        const embed = {
            type: 'rich',
            title: this.getEmbedTitle(usernames),
            description: TwitterUtil_1.TwitterUtil.getSpaceUrl(SpaceUtil_1.SpaceUtil.getId(this.audioSpace)),
            color: 0x1d9bf0,
            author: {
                name: `${name} (@${username})`,
                url: TwitterUtil_1.TwitterUtil.getUserUrl(username),
                icon_url: SpaceUtil_1.SpaceUtil.getHostProfileImgUrl(this.audioSpace),
            },
            fields,
            footer: {
                text: 'Twitter',
                icon_url: 'https://abs.twimg.com/favicons/twitter.2.ico',
            },
        };
        return embed;
    }
    static getEmbedLocalTime(ms) {
        if (!ms) {
            return null;
        }
        return [
            (0, builders_1.time)(Math.floor(ms / 1000)),
            (0, builders_1.time)(Math.floor(ms / 1000), 'R'),
        ].join('\n');
    }
}
exports.Webhook = Webhook;
//# sourceMappingURL=Webhook.js.map